// CoMET - The Crimson Fields Map Editing Tool
// Copyright (C) 2002-2003 Jens Granseuer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

////////////////////////////////////////////////////////////////////////
// mission.cpp
//
// History:
//  17-11-2002 - created
////////////////////////////////////////////////////////////////////////

#include "SDL_endian.h"

#include "mission.h"
#include "fileio.h"
#include "globals.h"

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::Mission
// DESCRIPTION: Create a new mission.
// PARAMETERS : size - map size
//              ts   - tile set to be used
//              us   - unit set to be used
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

Mission::Mission( const Point &size, TerrainSet *ts, UnitSet *us ) :
                  p1(PLAYER_ONE), p2(PLAYER_TWO) {
  flags = 0;
  level_info = -1;
  next_map = -1;
  msg_num = 0;
  msg_len = 0;
  password[0] = '\0';
  SetTitle( "Unknown" );

  unit_set = us;
  terrain_set = ts;

  map.SetUnitSet( unit_set );
  map.SetTerrainSet( terrain_set );
  map.SetSize( size );
}

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::~Mission
// DESCRIPTION: Destructor
// PARAMETERS : -
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

Mission::~Mission( void ) {
  delete unit_set;
  delete terrain_set;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::Load
// DESCRIPTION: Load a game from a mission file.
// PARAMETERS : filename - mission file name
// RETURNS    : 0 on success, non-zero otherwise
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Mission::Load( const char *filename ) {
  int rc = -1;
  SDL_RWops *file = SDL_RWFromFile( filename, "rb" );
  if ( !file ) return -1;

  // read game info
  unsigned char version;
  unsigned long id = SDL_ReadLE32( file );
  SDL_RWread( file, &version, 1, 1 );
  if ( (version == FILE_VERSION) && (id == FID_MISSION) ) {
    unsigned short len, i;
    unsigned char buf;
    char *uset, *tset;

    // set last filename
    last_file_name = file_part( filename );
    i = last_file_name.rfind( '.' );
    last_file_name.erase( i );

    flags = SDL_ReadLE16( file );
    i = SDL_ReadLE16( file );       // turn number - unused

    if ( flags & GI_SAVEFILE )      // refuse to load saved games
      goto out;
    else if ( flags & GI_PASSWORD ) {
      SDL_RWread( file, password, 1, 8 );
      crypt( password );
    } else password[0] = '\0';

    SDL_RWread( file, &level_info, 1, 1 );
    SDL_RWread( file, &next_map, 1, 1 );
    SDL_RWread( file, &buf, 1, 1 );     // current player - unused
    SDL_RWread( file, &buf, 1, 1 );     // turn phase - unused

    len = SDL_ReadLE16( file );        // load name of unit set
    uset = new char [len + 1];
    SDL_RWread( file, uset, 1, len );
    uset[len] = '\0';

    len = SDL_ReadLE16( file );        // load name of terrain set
    tset = new char [len + 1];
    SDL_RWread( file, tset, 1, len );
    tset[len] = '\0';

    map.Load( file );                  // load map

    unit_set = new UnitSet;
    rc = unit_set->Load( uset );
    delete [] uset;
    if ( rc ) {
      fprintf( stderr, "Error: Unit set not available.\n" );
      goto out;
    }

    terrain_set = new TerrainSet;
    rc = terrain_set->Load( tset );
    delete [] tset;
    if ( rc ) {
      fprintf( stderr, "Error: Terrain set not available.\n" );
      goto out;
    }

    map.SetUnitSet( unit_set );
    map.SetTerrainSet( terrain_set );

    p1.Load( file );
    p2.Load( file );
    if ( flags & GI_AI ) p2.SetType( COMPUTER );

    len = SDL_ReadLE16( file );         // load buildings
    for ( i = 0; i < len; ++i ) {
      Building *b = new Building();
      b->Load( file );
      buildings.AddTail( b );
      map.SetBuilding( b, b->Position() );
    }

    len = SDL_ReadLE16( file );         // load units
    for ( i = 0; i < len; ++i ) {
      unsigned char tid;
      SDL_RWread( file, &tid, 1, 1 );

      Unit *u = new Unit( file, unit_set->GetUnitInfo(tid) );
      if ( u ) {
        units.AddTail( u );
        if ( !map.GetMapObject(u->Position()) )
          map.SetUnit( u, u->Position() );
        else u->SetFlags( U_SHELTERED );
      }
    }

    len = SDL_ReadLE16( file );         // load combats

    // combat actions - only present in saved games

    len = SDL_ReadLE16( file );         // load events
    for ( i = 0; i < len; ++i ) {
      Event *e = new Event();
      e->Load( file );
      events.AddTail( e );
    }

    // load text messages
    msg_num = SDL_ReadLE16( file );             // number of messages
    msg_len = SDL_ReadLE32( file );
    if ( msg_num ) {                            // load messages
      char *ptr, *msgs = new char[msg_len];

      SDL_RWread( file, msgs, 1, msg_len );

      i = 0;
      ptr = msgs;
      while ( i < msg_num ) {
        crypt( ptr );
        messages.AddTail( new Message( ptr ) );
        ++i;

        while ( *ptr != '\0' ) ++ptr;
        ++ptr;
      }
      delete [] msgs;
    }

    len = SDL_ReadLE16( file );

    // history - only present in saved games

    rc = 0;
  }

out: 
  SDL_RWclose( file );
  return rc;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::Save
// DESCRIPTION: Save the mission to a file.
// PARAMETERS : filename - data file name
// RETURNS    : 0 on successful write, non-zero otherwise
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Mission::Save( const char *filename ) {
  SDL_RWops *file = SDL_RWFromFile( filename, "wb" );
  if ( !file ) {
    fprintf( stderr, "Failed to save mission!" );
    return -1;
  }

  unsigned char version = FILE_VERSION, current_player = PLAYER_ONE,
                turn_phase = TURN_START;
  unsigned short len, num;

  // set last filename
  last_file_name = file_part( filename );
  num = last_file_name.rfind( '.' );
  last_file_name.erase( num );

  // save game info
  SDL_WriteLE32( file, FID_MISSION );
  SDL_RWwrite( file, &version, 1, 1 );

  SDL_WriteLE16( file, flags );
  SDL_WriteLE16( file, 1 );        // turn

  SDL_RWwrite( file, &level_info, 1, 1 );
  SDL_RWwrite( file, &next_map, 1, 1 );
  SDL_RWwrite( file, &current_player, 1, 1 );
  SDL_RWwrite( file, &turn_phase, 1, 1 );

  len = unit_set->GetName().length();            // save mission set info
  SDL_WriteLE16( file, len );
  SDL_RWwrite( file, unit_set->GetName().c_str(), 1, len );
  len = terrain_set->GetName().length();
  SDL_WriteLE16( file, len );
  SDL_RWwrite( file, terrain_set->GetName().c_str(), 1, len );

  map.Save( file );            // save map

  p1.Save( file );              // save player data
  p2.Save( file );

  num = buildings.CountNodes();                 // save buildings
  SDL_WriteLE16( file, num );
  for ( Building *b = static_cast<Building *>( buildings.Head() );
        b; b = static_cast<Building *>( b->Next() ) ) b->Save( file );

  // save transports; basically, transports are not much different
  // from other units but we MUST make sure that transports having
  // other units on board are loaded before those units; we need to
  // make two passes through the list, first saving all filled
  // transports and those possibly filled, and all other units in
  // the second run
  num = units.CountNodes();
  SDL_WriteLE16( file, num ); 
  Unit *u;
  for ( u = static_cast<Unit *>( units.Head() );
        u; u = static_cast<Unit *>( u->Next() ) ) {       // save "free" transports
    if ( u->IsTransport() && !u->IsSheltered() ) u->Save( file );
  }

  for ( u = static_cast<Unit *>( units.Head() );
        u; u = static_cast<Unit *>( u->Next() ) ) {       // save other units
    if ( !u->IsTransport() || u->IsSheltered() ) u->Save( file );
  }

  num = 0;                                             // save combat data
  SDL_WriteLE16( file, num );

  num = events.CountNodes();                           // save events
  SDL_WriteLE16( file, num );
  for ( Event *e = static_cast<Event *>( events.Head() );
        e; e = static_cast<Event *>( e->Next() ) )
    e->Save( file );

  SDL_WriteLE16( file, msg_num );                     // save text messages
  SDL_WriteLE32( file, msg_len );
  for ( Message *m = static_cast<Message *>( messages.Head() );
        m; m = static_cast<Message *>( m->Next() ) ) m->Save( file );

  SDL_WriteLE16( file, 0 );                           // turn history

  SDL_RWclose( file );
  return 0;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::Export
// DESCRIPTION: Save the mission data to a plain text file.
// PARAMETERS : filename - save file name
// RETURNS    : 0 on success, -1 on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Mission::Export( const string &filename ) const {
  ofstream file( filename.c_str() );
  if ( !file ) return -1;

  string title( file_part(filename) );
  int suffix = title.rfind( '.' );
  title.erase( suffix );

  file << "[mission]\n";
  file << "title = " << title << '\n';
  file << "mapwidth = " << map.Width() << '\n';
  file << "mapheight = " << map.Height() << '\n';
  file << "info = " << (short)level_info << '\n';
  file << "nextmap = " << (short)next_map << '\n';
  file << "players = " << (((flags & GI_AI) != 0) ? '1' : '2') << '\n';
  if ( password[0] != '\0' )
    file << "password = " << password << '\n';
  file << "tileset = " << terrain_set->GetName() << '\n';
  file << "unitset = " << unit_set->GetName() << "\n\n";

  map.Export( file );
  p1.Export( file );
  p2.Export( file );

  Unit *u;
  for ( u = static_cast<Unit *>(units.Head()); u;
        u = static_cast<Unit *>(u->Next()) )
    if ( u->IsSheltered() || !u->IsTransport() ) u->Export( file );

  for ( u = static_cast<Unit *>(units.Head()); u;
        u = static_cast<Unit *>(u->Next()) )
    if ( !u->IsSheltered() && u->IsTransport() ) u->Export( file );

  for ( Building *b = static_cast<Building *>(buildings.Head()); b;
        b = static_cast<Building *>(b->Next()) )
    b->Export( file, unit_set );

  for ( Event *e = static_cast<Event *>(events.Head()); e;
        e = static_cast<Event *>(e->Next()) )
    e->Export( file, unit_set );

  if ( messages.CountNodes() > 0 ) {
    file << "[messages]";

    for ( Message *m = static_cast<Message *>(messages.Head()); m;
          m = static_cast<Message *>(m->Next()) ) {
      file << '\n';
      m->Export( file );
      file << '%';
    }
    file << "%\n\n";
  }

  file.close();
  return 0;
}


////////////////////////////////////////////////////////////////////////
// NAME       : Mission::CreateUnit
// DESCRIPTION: Create a new unit.
// PARAMETERS : type - unit type info
//              pid  - identifier of cotrolling player
//              pos  - position on map
// RETURNS    : created unit or NULL on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

Unit *Mission::CreateUnit( const UnitType *type, unsigned char pid, const Point &pos ) {
  Unit *u = new Unit( type, pid, GetUnitID(), pos );
  if ( u ) {
    units.AddTail( u );
    map.SetUnit( u, pos );
  }
  return u;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::DeleteUnit
// DESCRIPTION: Delete a unit, and for transporters also all freight.
// PARAMETERS : u - unit to remove from mission
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void Mission::DeleteUnit( Unit *u ) {
  if ( !u->IsSheltered() ) {
    map.SetUnit( NULL, u->Position() );
    u->Remove();

    if ( u->IsTransport() ) {
      Unit *next, *f = static_cast<Unit *>( units.Head() );
      while ( f ) {
        next = static_cast<Unit *>( f->Next() );

        if ( f->Position() == u->Position() ) {
          f->Remove();
          delete f;
        }

        f = next;
      }
    }
  }

  delete u;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::GetUnitByID
// DESCRIPTION: Find the unit corresponding to the given identifier.
// PARAMETERS : id - identifier of the unit to be searched for
// RETURNS    : pointer to the unit, or NULL if no unit with that ID
//              exists
//
// HISTORY
////////////////////////////////////////////////////////////////////////

Unit *Mission::GetUnitByID( unsigned short id ) const {
  Unit *u = static_cast<Unit *>( units.Head() );
  while ( u ) {
    if ( u->ID() == id ) return u;
    u = static_cast<Unit *>( u->Next() );
  }
  return NULL;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::GetUnitID
// DESCRIPTION: Get a unique identifier for a new unit.
// PARAMETERS : -
// RETURNS    : unique unit ID
//
// HISTORY
////////////////////////////////////////////////////////////////////////

unsigned short Mission::GetUnitID( void ) const {
  unsigned short id = 0;
  Unit *u = static_cast<Unit *>( units.Head() );

  while ( u ) {
    if ( u->ID() == id ) {
      ++id;
      u = static_cast<Unit *>( units.Head() );
    } else u = static_cast<Unit *>( u->Next() );
  }

  return id;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::CreateBuilding
// DESCRIPTION: Create a new building.
// PARAMETERS : type - unit type info
//              pid  - identifier of cotrolling player
//              pos  - position on map
// RETURNS    : created unit or NULL on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

Building *Mission::CreateBuilding( unsigned char pid, const Point &pos ) {
  Building *b = new Building( pos, GetBuildingID(), pid, "Depot" );
  if ( b ) {
    buildings.AddTail( b );
    map.SetBuilding( b, pos );
  }
  return b;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::DeleteBuilding
// DESCRIPTION: Delete a building and all units inside.
// PARAMETERS : b - building to remove from mission
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void Mission::DeleteBuilding( Building *b ) {
  map.SetBuilding( NULL, b->Position() );
  b->Remove();

  Unit *next, *f = static_cast<Unit *>( units.Head() );
  while ( f ) {
    next = static_cast<Unit *>( f->Next() );

    if ( f->Position() == b->Position() ) {
      f->Remove();
      delete f;
    }

    f = next;
  }

  delete b;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Mission::GetBuildingID
// DESCRIPTION: Get a unique identifier for a new building.
// PARAMETERS : -
// RETURNS    : unique building ID
//
// HISTORY
////////////////////////////////////////////////////////////////////////

unsigned short Mission::GetBuildingID( void ) const {
  unsigned short id = 0;
  Building *b = static_cast<Building *>( buildings.Head() );

  while ( b ) {
    if ( b->ID() == id ) {
      ++id;
      b = static_cast<Building *>( buildings.Head() );
    } else b = static_cast<Building *>( b->Next() );
  }

  return id;
}


////////////////////////////////////////////////////////////////////////
// NAME       : Player::Player
// DESCRIPTION: Create a new player.
// PARAMETERS : id - player identifier (PLAYER_ONE or PLAYER_TWO)
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

Player::Player( unsigned char id ) {
  p_id = id;
  p_type = HUMAN;
  p_level = 0;
  p_success = 0;
  p_briefing = -1;
  sprintf( p_name, "Player %d", id + 1 );
  for ( int i = 0; i < 8; ++i ) p_password[i] = '\0';
}

////////////////////////////////////////////////////////////////////////
// NAME       : Player::Load
// DESCRIPTION: Load player information from a file.
// PARAMETERS : file - data file descriptor
// RETURNS    : 0 on success, -1 on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Player::Load( SDL_RWops *file ) {
  SDL_RWread( file, &p_id, 1, 1 );
  SDL_RWread( file, &p_type, 1, 1 );
  SDL_RWread( file, &p_level, 1, 1 );
  SDL_RWread( file, &p_success, 1, 1 );
  SDL_RWread( file, &p_briefing, 1, 1 );
  SDL_RWread( file, &p_name, 1, 16 );
  SDL_RWread( file, &p_password, 1, 8 );
  return 0;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Player::Save
// DESCRIPTION: Save player information to a file.
// PARAMETERS : file - data file descriptor
// RETURNS    : 0 on success, non-zero on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Player::Save( SDL_RWops *file ) const {
  SDL_RWwrite( file, &p_id, 1, 1 );
  SDL_RWwrite( file, &p_type, 1, 1 );
  SDL_RWwrite( file, &p_level, 1, 1 );
  SDL_RWwrite( file, &p_success, 1, 1 );
  SDL_RWwrite( file, &p_briefing, 1, 1 );
  SDL_RWwrite( file, &p_name, 1, 16 );
  SDL_RWwrite( file, &p_password, 1, 8 );
  return 0;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Player::Export
// DESCRIPTION: Save the player to a plain text file.
// PARAMETERS : file - data file descriptor
// RETURNS    : 0 on success, -1 on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Player::Export( ofstream &file ) const {
  file << "[player]\n";
  file << "name = " << p_name << '\n';
  file << "briefing = " << (short)p_briefing << "\n\n";
  return 0;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Event::Event
// DESCRIPTION: Create a new event.
// PARAMETERS : type     - event type
//              trigger  - event trigger type
//              td1..td3 - trigger data
//              d1..d3   - event data
//              title    - message title string index
//              msg      - message string index
//              flags    - event flags
//              player   - player the event refers to
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

Event::Event( unsigned char type, unsigned char trigger,
         short td1, short td2, short td3, short d1, short d2, short d3,
         short title, short msg, unsigned short flags, unsigned char player ) {
  e_type = type;
  e_trigger = trigger;
  e_tdata[0] = td1;
  e_tdata[1] = td2;
  e_tdata[2] = td3;
  e_data[0] = d1;
  e_data[1] = d2;
  e_data[2] = d3;
  e_title = title;
  e_message = msg;
  e_flags = flags;
  e_player = player;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Event::Load
// DESCRIPTION: Load an event from a file.
// PARAMETERS : file    - file descriptor
// RETURNS    : 0 on success, -1 on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Event::Load( SDL_RWops *file ) {
  int i;

  SDL_RWread( file, &e_type, 1, 1 );
  SDL_RWread( file, &e_trigger, 1, 1 );
  for ( i = 0; i < 3; ++i ) e_tdata[i] = SDL_ReadLE16( file );
  for ( i = 0; i < 3; ++i ) e_data[i] = SDL_ReadLE16( file );
  e_title = SDL_ReadLE16( file );
  e_message = SDL_ReadLE16( file );
  e_flags = SDL_ReadLE16( file );
  SDL_RWread( file, &e_player, 1, 1 );
  return 0;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Event::Save
// DESCRIPTION: Save the event data to a file.
// PARAMETERS : file - data file descriptor
// RETURNS    : 0 on success, -1 on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Event::Save( SDL_RWops *file ) const {
  int i;

  SDL_RWwrite( file, &e_type, 1, 1 );
  SDL_RWwrite( file, &e_trigger, 1, 1 );
  for ( i = 0; i < 3; ++i ) SDL_WriteLE16( file, e_tdata[i] );
  for ( i = 0; i < 3; ++i ) SDL_WriteLE16( file, e_data[i] );
  SDL_WriteLE16( file, e_title );
  SDL_WriteLE16( file, e_message );
  SDL_WriteLE16( file, e_flags );
  SDL_RWwrite( file, &e_player, 1, 1 );
  return 0;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Event::Export
// DESCRIPTION: Save the event to a plain text file.
// PARAMETERS : file - data file descriptor
//              uset - current unit set
// RETURNS    : 0 on success, -1 on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Event::Export( ofstream &file, const UnitSet *uset ) const {
  const char *type[] = { "message", "production", "score",
                         "nextmap", "createunit" };

  file << "[event]\n";
  file << "type = " << type[e_type-1];
  if ( e_type == EVENT_PRODUCTION ) {
    if ( e_data[2] == 0 ) file << "(crystals)";
    else file << "(unit)";
  }
  file << "\nplayer = " << (e_player + 1) << '\n';
  if ( e_title != -1 )   file << "title = " << e_title << '\n';
  if ( e_message != -1 ) file << "message = " << e_message << '\n';
  if ( e_flags != 0 ) file << "flags = " << e_flags << '\n';

  file << "trigger = ";
  switch ( e_trigger ) {
  case ETRIGGER_TURN:
    file << "turn\n";
    file << "tturn = " << e_tdata[0] << '\n';
    break;
  case ETRIGGER_UNIT_DESTROYED:
    file << "unitdestroyed\n";
    file << "tunit = " << e_tdata[0] << '\n';
    file << "towner = " << e_tdata[1] + 1 << '\n';
    break;
  case ETRIGGER_HAVE_BUILDING:
    file << "havebuilding\n";
    file << "tbuilding = " << e_tdata[0] << '\n';
    file << "towner = " << e_tdata[1] + 1 << '\n';
    file << "tturn = " << e_tdata[2] << '\n';
    break;
  case ETRIGGER_HAVE_UNIT:
    file << "haveunit\n";
    file << "tunit = " << e_tdata[0] << '\n';
    file << "towner = " << e_tdata[1] + 1 << '\n';
    file << "tturn = " << e_tdata[2] << '\n';
    break;
  case ETRIGGER_UNIT_POSITION:
    file << "unitposition\n";
    file << "tunit = " << e_tdata[0] << '\n';
    file << "txpos = " << e_tdata[1] << '\n';
    file << "typos = " << e_tdata[2] << '\n';
    break;
  }

  switch ( e_type ) {
  case EVENT_PRODUCTION:
    file << "building = " << e_data[0] << '\n';
    if ( e_data[2] == 0 )
      file << "crystals = " << e_data[1] << '\n';
    else
      file << "unit = " << uset->GetUnitInfo(e_data[1])->Name() << '\n';
    break;
  case EVENT_SCORE:
    file << "success = " << e_data[0] << '\n';
    break;
  case EVENT_CREATE_UNIT:
    file << "unit = " << uset->GetUnitInfo(e_data[0])->Name() << '\n';
    file << "building = " << e_data[1] << '\n';
    break;
  case EVENT_NEXT_MAP:
    file << "map = " << e_data[0] << '\n';
    break;
  }

  file << '\n';
  return 0;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Message::Save
// DESCRIPTION: Save the message to a file.
// PARAMETERS : file - data file descriptor
// RETURNS    : 0 on success, -1 on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Message::Save( SDL_RWops *file ) const {
  char *crypted = new char[text.length() + 1];
  strcpy( crypted, text.c_str() );
  crypt( crypted );

  SDL_RWwrite( file, &crypted, 1, text.length() + 1 );
  delete [] crypted;
  return 0;
}

////////////////////////////////////////////////////////////////////////
// NAME       : Message::Export
// DESCRIPTION: Save the message to a plain text file.
// PARAMETERS : file - data file descriptor
// RETURNS    : 0 on success, -1 on error
//
// HISTORY
////////////////////////////////////////////////////////////////////////

int Message::Export( ofstream &file ) const {
  file << text << '\n';
  return 0;
}

